package cn.com.qmth.stmms.biz.mark.service.Impl;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

/**
 * 与评卷相关的全局状态锁<br>
 * 暂时只实现JVM内存锁，后续加上Redis实现的集群锁
 * 
 * @author luoshi
 *
 */
@Component
public class MarkLockService {

    protected static final Logger log = LoggerFactory.getLogger(MarkLockService.class);

    private Map<String, Boolean> groupMap = new ConcurrentHashMap<>();

    private Map<Integer, Boolean> studentMap = new ConcurrentHashMap<>();

    /**
     * 对评卷分组进行重置/删除操作前，先尝试锁定
     * 
     * @param group
     */
    public void lockGroup(Integer examId, String subjectCode, Integer groupNumber) {
        groupMap.put(getKey(examId, subjectCode, groupNumber), Boolean.TRUE);
    }

    /**
     * 对评卷分组进行重置/删除操作后，释放锁定
     * 
     * @param group
     */
    public void unlockGroup(Integer examId, String subjectCode, Integer groupNumber) {
        groupMap.remove(getKey(examId, subjectCode, groupNumber));
    }

    /**
     * 等待评卷分组释放锁定
     * 
     * @param group
     */
    public void waitUnlockGroup(Integer examId, String subjectCode, Integer groupNumber) {
        String key = getKey(examId, subjectCode, groupNumber);
        while (groupMap.get(key) != null) {
            ;
        }
    }

    /**
     * 对考生评卷任务进行删除操作前，先尝试锁定
     * 
     * @param studentId
     */
    public void lockStudent(Integer studentId) {
        studentMap.put(studentId, Boolean.TRUE);
    }

    /**
     * 对考生评卷任务进行删除操作后，释放锁定
     * 
     * @param studentId
     */
    public void unlockStudent(Integer studentId) {
        studentMap.remove(studentId);
    }

    /**
     * 等待考生释放锁定
     * 
     * @param studentId
     */
    public void waitUnlockStudent(Integer studentId) {
        while (studentMap.get(studentId) != null) {
            ;
        }
    }

    private String getKey(Integer examId, String subjectCode, Integer groupNumber) {
        return examId + "_" + subjectCode + "_" + groupNumber;
    }

}
